/* -*-  Mode:C; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1999, 2000 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the Linux source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CMAUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CMAUTHORS file.
 */
/*
 * An implementation of the Congestion Manager (CM) for Linux.  
 */
#ifndef _CM_APP_H
#define _CM_APP_H

/* User code visible API */

#include <sys/ioctl.h>
#include <netinet/in.h>

/* lossmode in cm_update() */
#define CM_NOLOSS     0
#define CM_PERSISTENT 1
#define CM_TRANSIENT  2
#define CM_ECN        3

/* needs to agree with linux/include/linux/socket.h */
#define SOCK_CM		6
#define IPPROTO_CM	6

/*used in setsockopt, agrees with include/linux/in.h */
#define CM_OPTIONS    15

/* CM stream type BUF/ALF, agrees with linux/include/net/cm.h */
#define CM_NO_BUF_ALF   0
#define CM_BUF  	1
#define CM_ALF 		2

/* CM rate callback enable */
#define CM_NO_RATE_CALLBACK 10
#define CM_RATE_CALLBACK 11

/* CM cong ctl algorithm */
#define AIMD 0
#define BINOMIAL 1

#define SIOCCMNOTIFY	SIOCPROTOPRIVATE
#define SIOCCMUPDATE	SIOCPROTOPRIVATE+1
#define SIOCCMREQUEST	SIOCPROTOPRIVATE+2
#define SIOCCMREADY	SIOCPROTOPRIVATE+3
#define SIOCCMQUERY	SIOCPROTOPRIVATE+4
#define SIOCCMBULKQUERY  SIOCPROTOPRIVATE+5
#define SIOCCMCONGPARAM  SIOCPROTOPRIVATE+6
#define SIOCCMGETMFLOW   SIOCPROTOPRIVATE+7
#define SIOCCMSETMFLOW   SIOCPROTOPRIVATE+8
#define SIOCCMGETTHRESHM         SIOCPROTOPRIVATE+9
#define SIOCCMSETTHRESHM         SIOCPROTOPRIVATE+10
#define SIOCCMGETASSOC         SIOCPROTOPRIVATE+11
#define SIOCCMSETASSOC         SIOCPROTOPRIVATE+12
#define SIOCCMBULKNOTIFY SIOCPROTOPRIVATE+13
#define SIOCCMBULKUPDATE SIOCPROTOPRIVATE+14
#define SIOCCMBULKREQUEST SIOCPROTOPRIVATE+15
#define SIOCCMDELASSOC   SIOCPROTOPRIVATE+17
#define SIOCCMSETCONGALG         SIOCPROTOPRIVATE+18


struct cmquery;
struct cmupdate;
struct cmthresh;

typedef struct cmquery {
	double rate;
	double srtt;
	double rttvar;
	int ratechange;
} cmquery;

typedef struct cmfdquery {
	int fd;
	struct cmquery cmquery;
	int ready;
} cmfdquery;

typedef struct cmupdate {
	int	nrecd;
	int	nloss;
	int	lossmode;
	unsigned int rtt;
} cmupdate;

typedef struct cmthresh {
	float rate_upmult;
	float rate_downmult;
	float rtt_upmult;
	float rtt_downmult;
} cmthresh;

extern int cm_open(struct sockaddr_in *srcaddr, struct sockaddr_in *dstaddr);
extern int cm_mtu(int id);
extern int cm_request(int id);
extern int cm_bulk_request(int id, int *list);
extern int cm_update(int id, int nrecd, int nlost, int lossmode,
			unsigned int rtt);
extern int cm_bulk_update(int id, int *list);
extern int cm_notify(int id, int nsent);
extern int cm_bulk_notify(int id, int *list);

extern int cm_query(int id, int *ready, cmquery *cmquery);
extern int cm_bulk_query(int id, cmfdquery *cmquery);
extern int cm_getmflow(int id);
extern int cm_setmflow(int id, int mflow);
extern int cm_getthresh(int id, cmthresh *cmthresh);
extern int cm_setthresh(int id, cmthresh *cmthresh);
extern int cm_getassoc(int id, int *list);
extern int cm_setassoc(int id, int *list);
extern int cm_close(int id);
extern int cm_setcongalg(int id, int type);

extern void cm_register_send(void (*handler)(int));
extern void cm_register_update(void (*handler)(int, cmquery));
extern void cm_dispatch(int fd, int select_read, int select_write, int select_except);

#endif
